#!/bin/sh

if grep -q QEMU /proc/cpuinfo || dmesg | grep -q 'Manufacturer: QEMU' || [ "${1}" = "force" ]; then
	for x in /dev/md/* ; do
		mdadm --stop $x
	done

	for disk in $(list-devices disk); do
		wipefs -a /dev/$disk[p][0-9]
		wipefs -a /dev/$disk
		mdadm --zero-superblock /dev/$disk
		dd if=/dev/zero of=/dev/$disk bs=512 count=32
		blkdiscard --secure /dev/${disk}
		parted -s /dev/${disk} mklabel gpt;
		partprobe
	done

	for x in /dev/md[0-9]* ; do
		mdadm --stop $x
	done
fi

RECIPEFILE=/tmp/expert_recipe
RAIDFILE=/tmp/raid_recipe
POSTFILE=/tmp/post-part.sh
POSTRAID=""
rm -f "${POSTFILE}"
rm -f "${RECIPEFILE}"
rm -f "${RAIDFILE}"
MBR_LIMIT=2254857830400
DISKS=""
RAIDS=""

debconfset() {
	: | debconf-set "${1}" "${2}"
}

getpart() {
	local partnum disk
	disk=${1}
	partnum=${2}
	if echo "${disk}" | grep -q nvme; then
		echo "${disk}p${partnum}"
	else
		echo "${disk}${partnum}"
	fi
}

get_raid_parts() {
	local partnum disks resstr del
	partnum=${1}
	shift
	disks=${@}
	for disk in ${disks}; do
		if [ -n "${resstr}" ]; then
			del="#"
		else
			del=""
		fi
		resstr="${resstr}${del}$(getpart ${disk} ${partnum})"
	done
	echo ${resstr}
}

prepare_serials() {
	local serial serial_scsi
	for disk in $(list-devices disk); do
		serial=$(udevadm info --query=property --name=${disk} | grep ID_SERIAL_SHORT | cut -d= -f2)
		serial_scsi=$(udevadm info --query=property --name=${disk} | grep ID_SCSI_SERIAL | cut -d= -f2)
		if [ -n "${serial}" ]; then
			DISKS="${DISKS} ${serial}:${disk}"
		fi
        if [ -n "${serial_scsi}" ] && ! echo "${serial_scsi}" | grep -q '[[:space:]]'; then
            DISKS="${DISKS} ${serial_scsi}:/dev/${disk}"
        fi
	done
}

prepare_raid() {
	local line raidname raidlevel hdd hdds
	line="${1}"
	set ${line}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			RAID_TYPE)
				raidlevel="${val##*_}"
			;;
			NAME)
				raidname="${val}"
			;;
			HDD_*)
				hdd=$(get_disk_device "${val}")
				if [ -n "${hdd}" ]; then
					if [ -n "${hdds}" ]; then
						hdds="${hdds}:${hdd}"
					else
						hdds="${hdd}"
					fi
				fi	
			;;
			DEV_*)
				hdd="${val}"
				if [ -n "${hdds}" ]; then
					hdds="${hdds}:${hdd}"
				else
					hdds="${hdd}"
				fi
			;;
		esac
	done
	RAIDS="${RAIDS} ${raidname}:${raidlevel}:${hdds}"
}

getval() {
	echo "${1##*=}"
}

do_part() {
	# $1 - part_string
	# echo $1
	local mpoint size fstype primary
	mpoint=$(echo "${1}" | cut -d, -f1)
	size=$(echo "${1}" | cut -d, -f2)
	fstype=$(echo "${1}" | cut -d, -f3)
	if [ "${mpoint}" = "/boot" ]; then
		primary="\$primary{ } \$bootable{ }"
	fi
	if [ "${mpoint}" = "swap" ]; then
		echo -n "${size} 512 ${size} linux-swap method{ swap } format{ } . " >> ${RECIPEFILE}
	elif [ -z "${size}" ]; then
		echo -n "1024 10000 1000000000 ${fstype} ${primary} method{ format } format{ } use_filesystem{ } filesystem{ ${fstype} } mountpoint{ ${mpoint} } . " >> ${RECIPEFILE}
	else
		echo -n "${size} 50 ${size} ${fstype} ${primary} method{ format } format{ } use_filesystem{ } filesystem{ ${fstype} } mountpoint{ ${mpoint} } . " >> ${RECIPEFILE}
	fi
}

do_post_label() {
	# $1 - drive
	local hdd
	hdd="${1}"
	echo "parted -s ${hdd} mklabel gpt" >> ${POSTFILE}
}

get_next_mdraid() {
	local cur
	cur=$(mdadm --examine --scan | cut -d' ' -f2 | grep -Eo '[0-9]+' | sort | tail -n1)
	if [ -z "${cur}" ]; then
		echo 0
	else
		echo $((cur + 1))
	fi
}

do_raid_post_part() {
	# $1 - part
	# $2 - index
	# $3 - raidlevel
	# $4 - hdds
	# $5 - last
	local mpoint size fstype hdds start end index mdadmnum parts raidlevel raiddev devcount
	parts=""  # Some bug in Debian 8
	hdds="${4}"
	test -z "${hdds}" && return 1
	mpoint=$(echo "${1}" | cut -d, -f1)
	size=$(echo "${1}" | cut -d, -f2)
	fstype=$(echo "${1}" | cut -d, -f3)
	index="${2}"
	raidlevel="${3}"
	start="${5}"
	devcount=$(echo "${hdds}" | wc -w)
	local part
	if [ -n "${size}" ]; then
		end=$((start + size))
	else
		end="100%"
	fi
	for hdd in ${hdds} ; do
		echo "parted -s ${hdd} mkpart primary ${start} ${end}" >> ${POSTFILE}
		part="$(getpart ${hdd} ${index})"
		parts="${parts} ${part}"
	done
	hostname=$(hostname)

	cat  >> ${POSTFILE} << 'EOF'

		get_next_mdraid() {
		    local cur
		    cur=$(mdadm --examine --scan | cut -d' ' -f2 | grep -Eo '[0-9]+' | sort | tail -n1)
		    if [ -z "${cur}" ]; then
		        echo 0
		    else
		        echo $((cur + 1))
		    fi
		}
	
EOF
	echo "mdadmnum=\$(get_next_mdraid)" >> ${POSTFILE}
	echo "raiddev=\"/dev/md\${mdadmnum}\"" >> ${POSTFILE}
	echo "mdadm --create \${raiddev} --level=${raidlevel} --name=${hostname}:\${mdadmnum} --force --run --raid-devices=${devcount} ${parts}" >> ${POSTFILE}
	echo "mkfs.${fstype} \${raiddev}" >> ${POSTFILE}
	echo "if [ -b "\${raiddev}" ]; then" >> ${POSTFILE}
	echo -e "  echo \"\${raiddev}\t${mpoint}\t${fstype}\tdefaults\t0 0\" >> /target/etc/fstab" >> ${POSTFILE}
	echo "  if [ -s "/target/${mpoint}" ]; then mkdir -p /mnt/data; mount \${raiddev} /mnt/data ; mv /target/${mpoint}/* /mnt/data/ ; umount /mnt/data ; mount \${raiddev} /target/${mpoint} ; fi" >> ${POSTFILE}
	echo "  uuid=\$(mdadm --detail \${raiddev} | grep UUID | sed -r 's|\s*UUID\s*:\s*||')" >> ${POSTFILE}
	echo "  mdadm --examine --scan | grep UUID=\${uuid} >> /target/etc/mdadm/mdadm.conf" >> ${POSTFILE}
	echo "fi" >> ${POSTFILE}
	echo ${end}
}

do_post_part() {
	# $1 - drive
	# $2 - part
	# $3 - index
	# $4 - last
	local mpoint size fstype hdd start end index
	hdd="${1}"
	test -z "${hdd}" && return 1
	mpoint=$(echo "${2}" | cut -d, -f1)
	size=$(echo "${2}" | cut -d, -f2)
	fstype=$(echo "${2}" | cut -d, -f3)
	index="${3}"
	start="${4}"
	local part
	part="$(getpart ${hdd} ${index})"
	if [ -n "${size}" ]; then
		end=$((start + size))
	else
		end="100%"
	fi
	echo "parted -s ${hdd} mkpart primary ${start} ${end}" >> ${POSTFILE}
	echo "mkfs.${fstype} ${part}" >> ${POSTFILE}
	echo -e "if [ -b "${part}" ]; then echo '${part}\t${mpoint}\t${fstype}\tdefaults\t0 0' >> /target/etc/fstab; fi" >> ${POSTFILE}
	echo "if [ -s "/target/${mpoint}" ] && [ -b "${part}" ]; then mkdir -p /mnt/data; mount ${part} /mnt/data ; mv /target/${mpoint}/* /mnt/data/ ; umount /mnt/data ; mount ${part} /target/${mpoint} ; fi" >> ${POSTFILE}
	echo ${end}
}

do_raid_part() {
	# $1 - part_string
	# $2 - part_number
	# $3 - raid level
	# $4 - hdds
	local mpoint size fstype pindex raidlevel devcount hdds
	pindex="${2}"
	raidpindex=$((pindex + 1)) # Hidden efi or biosboot part
	raidlevel="${3}"
	mpoint=$(echo "${1}" | cut -d, -f1)
	size=$(echo "${1}" | cut -d, -f2)
	if [ -n "${size}" ] && [ ${size} -lt 10 ]; then
        size=10
	fi
	fstype=$(echo "${1}" | cut -d, -f3)
	hdds="${4}"
	devcount=$(echo "${hdds}" | wc -w)
	if [ -n "${size}" ]; then
		echo -n "${size} 50 ${size} raid \$primary{ } method{ raid } . " >> ${RECIPEFILE}
	else
		echo -n "1024 10000 1000000000 raid \$primary{ } method{ raid } . " >> ${RECIPEFILE}
	fi
	if [ "${mpoint}" = "swap" ]; then
		mpoint="-"
	fi
	echo -n "${raidlevel} ${devcount} 0 ${fstype} ${mpoint} $(get_raid_parts ${raidpindex} ${hdds}) . " >> ${RAIDFILE}
}

get_disk_size() {
	fdisk -l "${1}" 2>/dev/null | grep Disk | grep dev | cut -d' ' -f5
}


get_disk_device() {
	local dev name serial
	name=${1}
	for part in ${DISKS}; do
		serial=${part%:*}	
		if [ "${serial}" = "${name}" ]; then
			dev=${part#*:}
			break
		fi
	done
	echo ${dev}
}

do_system_parts() {
	local name hdds
	name="${1}"
    hdds="${2}"
	echo -n "${name} :: " >> ${RECIPEFILE}
	if [ -d /sys/firmware/efi ]; then
		echo -n "300 200 500 free \$iflabel{ gpt } \$reusemethod{ } method{ efi } format{ } . " >> ${RECIPEFILE}
        for hdd in ${hdds}; do
            if [ ! -f /tmp/first_efi_part ]; then
                getpart ${hdd} 1 >> /tmp/first_efi_part
            else
                getpart ${hdd} 1 >> /tmp/second_efi_parts
            fi
        done
	else
		echo -n "1 1 1 free \$iflabel{ gpt } method{ biosgrub } . " >> ${RECIPEFILE}
	fi
}

do_layout_hdd() {
	local line name val
	local dev install_os parts reciepestr
	# echo "${line}"
	# echo ${@}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			DEV)
				dev=${val}
			;;
			NAME)
				dev=$(get_disk_device "${val}")
			;;
			PART_*)
				part=${val#\"}
				part=${part%\"}
				#echo ${part}
				parts="${parts} ${part}"
			;;
			INSTALL_OS)
				install_os=true
			;;
		esac
	done
	
	if [ -n "${install_os}" ]; then
		debconf-set partman-auto/disk "${dev}"
		debconf-set partman-auto/method "regular"
		debconf-set grub-installer/bootdev "${dev}"
		
		do_system_parts "boot-root" "${dev}"
		for part in ${parts} ; do
			do_part "${part}"
		done
		
		recipestr="$(cat ${RECIPEFILE})"
		echo "${recipestr}" >> /tmp/custom.log
		debconf-set partman-auto/expert_recipe "${recipestr}"
	fi
	if [ -z "${install_os}" ]; then
		# Create post script
		# I think it is better than parse layout again in post-install step
		local last
		do_post_label "${dev}"
		pindex=0
		last=1
		for part in ${parts} ; do
			pindex=$((pindex + 1))
			last=$(do_post_part "${dev}" "${part}" "${pindex}" "${last}")
		done
	fi

}

get_raid_info() {
	local raidstr name lname
	name=${1}
	for raid in ${RAIDS} ; do
		lname=${raid%%:*}
		if [ "${lname}" = "${name}" ]; then
			raidstr=${raid#*:}
			break
		fi
	done
	echo "${raidstr}"
}

do_layout_raid() {
	local line name val
	local install_os raid raidstr raidlevel hdds parts pindex recipestr raidrecipe
	# echo "${line}"
	# echo ${@}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			NAME)
				raidstr=$(get_raid_info "${val}")
				raidlevel=${raidstr%%:*}
				hdds=$(echo "${raidstr#*:}" | sed 's|:| |g')
				devcount=$(echo "${hdds}" | wc -w)
			;;
			PART_*)
				part=${val#\"}
				part=${part%\"}
				#echo ${part}
				parts="${parts} ${part}"
			;;
			INSTALL_OS)
				install_os=true
			;;
		esac
	done
	if [ -n "${install_os}" ] && [ -z "${POST}" ]; then
		debconf-set partman-auto/disk "${hdds}"
		debconf-set grub-installer/bootdev "${hdds}"
		debconf-set partman-auto/method "raid"
	
		do_system_parts "multiraid" "${hdds}"

		echo "${hdds}" >> /tmp/grub_devices

		pindex=0
		for part in ${parts} ; do
			pindex=$((pindex + 1))
			do_raid_part "${part}" "${pindex}" "${raidlevel}" "${hdds}"
		done

		recipestr="$(cat ${RECIPEFILE})"
		echo "${recipestr}" >> /tmp/custom.log
		debconf-set partman-auto/expert_recipe "${recipestr}"

		raidrecipe="$(cat ${RAIDFILE})"
		echo "${raidrecipe}" >> /tmp/custom.log
		debconf-set partman-auto-raid/recipe "${raidrecipe}"
	fi
	if [ -z "${install_os}" ]; then
		# Create post script
		# I think it is better than parse layout again in post-install step
		for dev in ${hdds} ; do
			do_post_label "${dev}"
		done
		pindex=0
		last=1
		for part in ${parts} ; do
			echo "Do raid post part ${part}"
			pindex=$((pindex + 1))
			last=$(do_raid_post_part "${part}" "${pindex}" "${raidlevel}" "${hdds}" "${last}")
		done
		POSTRAID=yes
	fi
}

do_layout() {
	local line valname
	line="${1}"
	set ${line}
	#echo ${1}
	#echo ${@}
	for val in ${line} ; do
		# echo ${val}
		valname=${val%%=*}
		# echo $valname
		case ${valname} in
			TYPE)
				type=$(getval ${val})
				line=$(echo "${line}" | sed "s|${val}||")
				break
		esac
	done
	case ${type} in
		hdd)
			do_layout_hdd ${line}
		;;
		raid)
			do_layout_raid ${line}
		;;
	esac
}

prepare_serials

if [ -f /tmp/diskpart.txt ]; then
	echo "" >> /tmp/diskpart.txt
    sed -i -r '/^$/d' /tmp/diskpart.txt # trim
fi
IFS="
"
for line in $(cat /tmp/diskpart.txt); do
    unset IFS
	line_type=$(echo "${line}" | cut -d: -f1)
	line_content=$(echo "${line}" | cut -d: -f2-| sed 's|^\s||')
	case ${line_type} in
		raid)
			prepare_raid "${line_content}"
		;;
		layout)
			# Do layout
			do_layout "${line_content}"
		;;
	esac

done
unset IFS

if [ -n "${POSTRAID}" ]; then
	echo "in-target apt-get -y --no-install-recommends install mdadm" >> ${POSTFILE}
	echo "in-target update-initramfs -u" >> ${POSTFILE}
	echo "cp /target/etc/mdadm/mdadm.conf /target/root" >> ${POSTFILE}
fi

plabel=gpt
debconf-set partman-basicfilesystems/choose_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}
debconf-set partman-basicfilesystems/default_label ${plabel}
debconf-set partman-partitioning/default_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}

debconf-set partman/exception_handler Yes
