#!/bin/sh

for x in /dev/md/* ; do
	mdadm --stop $x
done

for disk in $(list-devices disk); do
	wipefs -a /dev/$disk[p][0-9]
	wipefs -a /dev/$disk
	mdadm --zero-superblock /dev/$disk
	dd if=/dev/zero of=/dev/$disk bs=512 count=32
	blkdiscard --secure /dev/${disk}
	parted -s /dev/${disk} mklabel gpt;
	partprobe
done

for x in /dev/md[0-9]* ; do
	mdadm --stop $x
done

set $(list-devices disk)
let numb=$#/2
DISKA=$1
DISKB=$2
SIZE1=$(fdisk -l "${DISKA}" 2>/dev/null|grep Disk|grep dev|cut -d' ' -f5)
MBR_LIMIT=$((2 * $((1000 ** 4))))

if [ ${SIZE1} -gt ${MBR_LIMIT} ]; then
	gpt=yes
fi
if [ -n "${gpt}" ] || [ -d /sys/firmware/efi ]; then
	# GPT partitioning
	plabel=gpt
	gpt=yes
else
	plabel=msdos
fi

debconf-set partman-basicfilesystems/choose_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}
debconf-set partman-basicfilesystems/default_label ${plabel}
debconf-set partman-partitioning/default_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}
	    
debconf-set partman-auto/disk "$DISKA"
debconf-set partman-auto/method "regular"

bootsize=256

if [ -d /sys/firmware/efi ]; then
  debconf-set partman-auto/expert_recipe "boot-root :: \
    300 200 500 free $iflabel{ gpt } $reusemethod{ } method{ efi } format{ } . \
    ${bootsize} 50 ${bootsize} ext2 $primary{ } $bootable{ } method{ format } format{ } use_filesystem{ } filesystem{ ext2 } mountpoint{ /boot } . \
    128 512 100% linux-swap method{ swap } format{ } . \
    1024 10000 1000000000 ext4 method{ format } format{ } use_filesystem{ } filesystem{ ext4 } mountpoint{ / } . "
elif [ -n "${gpt}" ]; then
  debconf-set partman-auto/expert_recipe "boot-root :: \
    1 1 1 free $iflabel{ gpt } method{ biosgrub } . \
    ${bootsize} 50 ${bootsize} ext2 $primary{ } $bootable{ } method{ format } format{ } use_filesystem{ } filesystem{ ext2 } mountpoint{ /boot } . \
    128 512 100% linux-swap method{ swap } format{ } . \
    1024 10000 1000000000 ext4 method{ format } format{ } use_filesystem{ } filesystem{ ext4 } mountpoint{ / } . "
else
  debconf-set partman-auto/expert_recipe "boot-root :: \
    ${bootsize} 50 ${bootsize} ext2 $primary{ } $bootable{ } method{ format } format{ } use_filesystem{ } filesystem{ ext2 } mountpoint{ /boot } . \
    128 512 100% linux-swap method{ swap } format{ } . \
    1024 10000 1000000000 ext4 method{ format } format{ } use_filesystem{ } filesystem{ ext4 } mountpoint{ / } . "
fi

debconf-set grub-installer/bootdev "${DISKA}"
echo "${DISKA}" >> /tmp/grub_devices
