#!/bin/sh

for x in /dev/md/* ; do
	mdadm --stop $x
done

for disk in $(list-devices disk); do
	wipefs -a /dev/$disk[p][0-9]
	wipefs -a /dev/$disk
	mdadm --zero-superblock /dev/$disk
	dd if=/dev/zero of=/dev/$disk bs=512 count=32
	blkdiscard --secure /dev/${disk}
	parted -s /dev/${disk} mklabel gpt;
	partprobe
done

for x in /dev/md[0-9]* ; do
	mdadm --stop $x
done

set $(list-devices disk)
let numb=$#/2
DISKA=$1
DISKB=$2
SIZE1=$(fdisk -l "${DISKA}" 2>/dev/null|grep Disk|grep dev|cut -d' ' -f5)
MBR_LIMIT=$((2 * $((1000 ** 4))))

if [ ${SIZE1} -gt ${MBR_LIMIT} ]; then
	gpt=yes
fi
if [ -n "${gpt}" ] || [ -d /sys/firmware/efi ]; then
	# GPT partitioning
	plabel=gpt
	gpt=yes
else
	plabel=msdos
fi

getpart() {
	local partnum disk
	disk=${1}
	partnum=${2}
	if echo "${disk}" | grep -q nvme; then
		echo "${disk}p${partnum}"
	else
		echo "${disk}${partnum}"
	fi
}

get_raid_parts() {
	local partnum disks resstr del
	partnum=${1}
	shift
	disks=${@}
	for disk in ${disks}; do
		if [ -n "${resstr}" ]; then
			del="#"
		else
			del=""
		fi
		resstr="${resstr}${del}$(getpart ${disk} ${partnum})"
	done
	echo ${resstr}
}

debconf-set partman-basicfilesystems/choose_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}
debconf-set partman-basicfilesystems/default_label ${plabel}
debconf-set partman-partitioning/default_label ${plabel}
debconf-set partman-partitioning/choose_label ${plabel}
	    
debconf-set partman-auto/disk "$DISKA $DISKB"
debconf-set partman-auto/method "raid"


bootsize=256

if [ -d /sys/firmware/efi ]; then
  debconf-set partman-auto/expert_recipe "multiraid :: \
    300 200 500 free \$iflabel{ gpt } \$reusemethod{ } method{ efi } format{ } . \
    ${bootsize} 50 ${bootsize} raid \$primary{ } method{ raid } . \
    128 512 100% raid method{ raid } . \
    1024 10000 1000000000 raid method{ raid } . "
elif [ -n "${gpt}" ]; then
  debconf-set partman-auto/expert_recipe "multiraid :: \
    1 1 1 free \$iflabel{ gpt } method{ biosgrub } . \
    ${bootsize} 50 ${bootsize} raid \$primary{ } method{ raid } . \
    128 512 100% raid method{ raid } . \
    1024 10000 1000000000 raid method{ raid } . "
else
  debconf-set partman-auto/expert_recipe "multiraid :: \
    ${bootsize} 50 ${bootsize} raid \$primary{ } method{ raid } . \
    128 512 100% raid method{ raid } . \
    1024 10000 1000000000 raid method{ raid } . "
fi

if [ -n "${gpt}" ]; then
	debconf-set partman-auto-raid/recipe " \
	  1 2 0 ext4 /boot $(get_raid_parts 2 ${DISKA} ${DISKB}) . \
	  0 2 0 swap - $(get_raid_parts 3 ${DISKA} ${DISKB}) . \
	  0 2 0 ext4 / $(get_raid_parts 4 ${DISKA} ${DISKB}) . "
else
	debconf-set partman-auto-raid/recipe " \
	  1 2 0 ext4 /boot $(get_raid_parts 1 ${DISKA} ${DISKB}) . \
	  0 2 0 swap - $(get_raid_parts 5 ${DISKA} ${DISKB}) . \
	  0 2 0 ext4 / $(get_raid_parts 6 ${DISKA} ${DISKB}) . "
fi

debconf-set grub-installer/bootdev "${DISKA} ${DISKB}"
echo "${DISKA} ${DISKB}" >> /tmp/grub_devices
